
/*-------------------------------------------------------------
  module: toesacQXBLD.h
  author: Angel L. Diaz

  date: 12/23/96

  description: 
     Construct the factor black box of an nXn symetric toeplitz determinant 
     utilizing SACLIB's rational polynomial arithmetic 

  arguments:
     N       - size of toeplitz matrix
     DegDet  - bound on the degree of the determiant
     DegVar  - bound on the degree of the variables

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include "PlugIns/saclib.h"
#include "BlackBox/CommonObjects/bbtoeplitz.h"
#include "BlackBox/Aux/bbselector.h"
#include "BlackBox/Aux/bbvector.h"
#include "BlackBox/Algorithms/bbfactors.h"
#include "BlackBox/Algorithms/bbsparse.h"
#include "BlackBox/Aux/bbtime.h"

typedef BlackBoxSymToeDet< SaclibQ, SaclibQX >            BBSymToeDet;
typedef BlackBoxFactors< SaclibQ, SaclibQX, BBSymToeDet > BBFactors;

int main( int argc, char *argv[] ) 
{
   Word stack;
   SaclibInitEnv(3000000,stack);
   int    i,N,DegDet,Seed;
   double Prob;
   char   FileName[BBDEFS_FILEZ];
   char   ProbName[BBDEFS_FILEZ];
   MP_INT MPCard;

   if ( argc < 3 ) {
      cerr << "usage: n deg_det" << endl;
      SaclibCleanUpEnv();
      exit( -1 );
   }
   else {
      mpz_init_set_ui( &MPCard, 100000007 ); 
      Prob   = 1.0;
      Seed   = 1;
      N      = atoi( argv[1] );
      DegDet = atoi( argv[2] );

      sprintf( ProbName, "%dx%d%s", N, N, "toesacQXBLD" );
      GetCPUTime( ProbName );

      BBSymToeDet T( N, DegDet , Prob );
      BBFactors   FactorsT1( T, Prob, Seed, &MPCard );

      sprintf( FileName, "%s.ins", ProbName );
      ofstream OutFile( FileName, ios::out );
      if ( !OutFile ) {
         cerr << "toefacshmap : cannot open " << FileName
              << " for output" << endl;
         exit(-1);
      }

      OutFile << FactorsT1;
      OutFile.close();
      
      mpz_clear( &MPCard );
   }

   SaclibCleanUpEnv();
   GetCPUTime( ProbName );
}

