
/*-------------------------------------------------------------
  module: toesacQXT.h
  author: Angel L. Diaz

  date: 11/27/96

  description: 
     Interpolates a factor of an nXn symetric toeplitz determinant 
     utilizing SACLIB's truncated rational field and polynomial arithmetic 

  arguments:
     N       - size of toeplitz matrix
     DegDet  - bound on the degree of the determiant
     DegVar  - bound on the degree of the variables

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include "PlugIns/saclib.h"
#include "BlackBox/Aux/bbselector.h"
#include "BlackBox/Aux/bbvector.h"
#include "BlackBox/CommonObjects/bbpoly.h"
#include "BlackBox/CommonObjects/bbdet.h"
#include "BlackBox/Algorithms/bbfactors.h"
#include "BlackBox/Algorithms/bbsparse.h"
#include "BlackBox/Aux/bbtime.h"

typedef BlackBoxUPolyEx< SaclibQ, SaclibQXT >             BBUPSacQSacQXT;
typedef BlackBoxFactorsEx< SaclibQ, SaclibQX, 
        SaclibQXT, BBUPSacQSacQXT >                       BBFactors;
typedef BlackBoxSelectorEx< SaclibQ, SaclibQXT, 
        BBFactors >                                       BBFactor;

SaclibQXT SymToe4x4SacQXT( SaclibQXT* );
SaclibQ   SymToe4x4SacQ( SaclibQ* );

int main(  ) 
{
   Word stack;
   SaclibInitEnv(3000000,stack);
   SaclibSetTruncDeg( 5 );
   int    i,N, DegDet, DegVar,Fac,Seed,IsRestart;
   double Prob;
   MP_INT MPCard;
   mpz_init_set_si( &MPCard, 1000 ); 
   char   ProbName[BBDEFS_FILEZ];

   Prob      = 1.0;
   Seed      = 1;
   N         = 4;
   DegDet    = 4;
   DegVar    = 4;
   Fac       = 0;
   IsRestart = 0;
 
   sprintf( ProbName, "%dx%d%s", N, N, "toesacQXT" );
   GetCPUTime( ProbName );

   // can evaluate over Q and Q[Y] mod Y^d
   BBUPSacQSacQXT BBSymToe4x4SacQXT( SymToe4x4SacQ, SymToe4x4SacQXT, N, 
      DegDet, Prob );

   BBFactors BBFacsSymToe4x4SacQXT( BBSymToe4x4SacQXT, Prob, Seed, &MPCard );
   BBFactor  BBFacSymToe4x4SacQXT( BBFacsSymToe4x4SacQXT, 0 );

   BBSPARSEAUX_VarVector Vars;
   BBSPARSEAUX_DegVector  Degs;
   for( i=1; i<=N; i++ ) Vars.push_back(i);
   for( i=1; i<=N; i++ ) Degs.push_back(DegVar); 
   SaclibQX                 SaclibQXElm[BBSPARSEAUX_MAXTERMS];
   BBSPARSEAUX_VecDegVector PruneDegs;
   bbvector< SaclibQ >      PruneMons;

   SparseInterp( BBFacSymToe4x4SacQXT, Vars, Degs, DegDet, &MPCard, 
                 PruneDegs, PruneMons, SaclibQXElm, ProbName, IsRestart );

   mpz_clear( &MPCard );
   SaclibCleanUpEnv();
   GetCPUTime( ProbName );
}

SaclibQXT SymToe4x4SacQXT(
   SaclibQXT* Pnts_i
   )
{
  int r,c;
  SaclibQXT A[BBDET_MAXN][BBDET_MAXM];

  for( r=0; r<4; r++ ) {
     for( c=0; c<4; c++ ) {
        if ( c >= r )
           A[r][c] = Pnts_i[3-c+r];
        else
           A[r][c] = Pnts_i[3+c-r];
     }
  }
  return ( FracFreeElm( A, 4, 4 ) );
}

SaclibQ SymToe4x4SacQ(
   SaclibQ* Pnts_i
   )
{
  int r,c;
  SaclibQ A[BBDET_MAXN][BBDET_MAXM];

  for( r=0; r<4; r++ ) {
     for( c=0; c<4; c++ ) {
        if ( c >= r )
           A[r][c] = Pnts_i[3-c+r];
        else
           A[r][c] = Pnts_i[3+c-r];
     }
  }
  return ( FracFreeElm( A, 4, 4 ) );
}

