
/*-------------------------------------------------------------
  module: vandsacQX.h
  author: Angel L. Diaz

  date: 12/5/96

  description: 
     Interpolates the GCD of two 2*n variables vandermonde matrix
     determinants which differ in two variables utilizing SACLIB's 
     rational field and polynomial arithmetic 

  arguments:
     N       - size of toeplitz matrix
     DegDet  - bound on the degree of the determiant
     DegVar  - bound on the degree of the variables

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include "PlugIns/saclib.h"
#include "BlackBox/CommonObjects/bbvandermonde.h"
#include "BlackBox/Algorithms/bbgcd.h"
#include "BlackBox/Aux/bbvector.h"
#include "BlackBox/Algorithms/bbsparse.h"
#include "BlackBox/Aux/bbtime.h"

typedef BlackBoxVandDet< SaclibQ >                       BBVandDetSacQX;
typedef BlackBoxGCD< SaclibQ, SaclibQX, BBVandDetSacQX >  BBGCDSacQX;

int main( int argc, char *argv[] ) 
{
   Word stack;
   SaclibInitEnv(4000000,stack);
   int    i,N, r, DegDet, DegGCD, UsedVars,Seed,StartPnt,EndPnt,IsRestart;
   double Prob;
   MP_INT MPCard;
   char   ProbName[BBDEFS_FILEZ];

   if ( argc < 3 ) {
      cerr << "usage: n restart" << endl;
      SaclibCleanUpEnv();
      exit( -1 );
   }
   else {
      mpz_init_set_ui( &MPCard, 100000007 ); 
      Prob      = 1.0;
      Seed      = 1;
      N         = atoi( argv[1] );
      IsRestart = atoi( argv[2] );
      DegDet    = -1;
      UsedVars  = N/2;
      StartPnt  = N/2;
      EndPnt    = N;
      r         = 2;
      DegGCD    = 1;
      sprintf( ProbName, "%d%s", N, "vandsacQX" );
      GetCPUTime( ProbName );

      BBVandDetSacQX  BBVandDet1( N, DegDet, Prob, UsedVars );
      BBVandDetSacQX  BBVandDet2( N, DegDet, Prob, UsedVars, StartPnt,
                                  EndPnt, BBDEFS_TRUE );
      BBVandDetSacQX* BBArray[2] = { &BBVandDet1, &BBVandDet2 };

      BBGCDSacQX BBGCD( BBArray, r, &MPCard, Prob, Seed );

      BBSPARSEAUX_VarVector  Vars;
      BBSPARSEAUX_DegVector  Degs;
      for( i=1; i<=N; i++ ) Vars.push_back(i);
      Degs.push_back(1); Degs.push_back(1); 
      for( i=3; i<=N; i++ ) Degs.push_back(0); 
      SaclibQX  SaclibQXElm[BBSPARSEAUX_MAXTERMS];
      BBSPARSEAUX_VecDegVector PruneDegs;
      bbvector< SaclibQ >      PruneMons;


      SparseInterp( BBGCD, Vars, Degs, DegGCD, &MPCard, 
                    PruneDegs, PruneMons, SaclibQXElm, ProbName, IsRestart );

      mpz_clear( &MPCard );
   }

   SaclibCleanUpEnv();
   GetCPUTime( ProbName );
}

