
/*-------------------------------------------------------------
  module: vandsacZPX.h
  author: Angel L. Diaz

  date: 12/5/96

  description: 
     Interpolates the GCD of two 2*n variables vandermonde matrix
     determinants which differ in two variables utilizing SACLIB's 
     modular field and polynomial arithmetic 

  arguments:
     N       - size of toeplitz matrix

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include "PlugIns/saclib.h"
#include "BlackBox/CommonObjects/bbvandermonde.h"
#include "BlackBox/Algorithms/bbgcd.h"
#include "BlackBox/Aux/bbvector.h"
#include "BlackBox/Algorithms/bbsparse.h"
#include "BlackBox/Aux/bbtime.h"

typedef BlackBoxVandDet< SaclibZP >                          BBVandDetSacZPX;
typedef BlackBoxGCD< SaclibZP, SaclibZPX, BBVandDetSacZPX >  BBGCDSacZPX;

int main( int argc, char *argv[] ) 
{
   Word stack;
   SaclibInitEnv(1000000,stack);
   int    i,N, r, DegDet, DegGCD, UsedVars,Seed,StartPnt,EndPnt,IsRestart;
   double Prob;
   MP_INT MPCard;
   char   ProbName[BBDEFS_FILEZ];

   if ( argc < 3 ) {
      cerr << "usage: n restart" << endl;
      SaclibCleanUpEnv();
      exit( -1 );
   }
   else {
      mpz_init_set_ui( &MPCard, 100000007 ); 
      SaclibSetPrime( &MPCard );
      Prob      = 1.0;
      Seed      = 12234;
      N         = atoi( argv[1] );
      DegDet    = -1;
      UsedVars  = N/2;
      StartPnt  = N/2;
      EndPnt    = N;
      r         = 2;
      DegGCD    = 1;
      IsRestart = atoi( argv[2] );
      sprintf( ProbName, "%d%s", N, "vandsacZPX" );
      GetCPUTime( ProbName );

      BBVandDetSacZPX  BBVandDet1( N, DegDet, Prob, UsedVars );
      BBVandDetSacZPX  BBVandDet2( N, DegDet, Prob, UsedVars, StartPnt,
                                  EndPnt, BBDEFS_TRUE );
      BBVandDetSacZPX* BBArray[2] = { &BBVandDet1, &BBVandDet2 };

      BBGCDSacZPX BBGCD( BBArray, r, &MPCard, Prob, Seed );

      BBSPARSEAUX_VarVector  Vars;
      BBSPARSEAUX_DegVector  Degs;
      for( i=1; i<=N; i++ ) Vars.push_back(i);
      Degs.push_back(1); Degs.push_back(1); 
      for( i=3; i<=N; i++ ) Degs.push_back(0); 
      SaclibZPX  SaclibZPXElm[BBSPARSEAUX_MAXTERMS];
      BBSPARSEAUX_VecDegVector PruneDegs;
      bbvector< SaclibZP >     PruneMons;

      SparseInterp( BBGCD, Vars, Degs, DegGCD, &MPCard, 
                    PruneDegs, PruneMons, SaclibZPXElm, ProbName, IsRestart );

      mpz_clear( &MPCard );
   }

   SaclibCleanUpEnv();
   GetCPUTime( ProbName );
}

