
/*-------------------------------------------------------------
  module: ratfuncsacQX.C
  author: Angel L. Diaz

  date: 12/5/96

  description: 
     Interpolates the numerator of a rational function (x^2+y) 
     utilizing SACLIB's rational field and polynomial arithmetic.

  arguments:
     none

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include "PlugIns/saclib.h"
#include "BlackBox/CommonObjects/bbratfunc.h"
#include "BlackBox/Algorithms/bbnumden.h"
#include "BlackBox/Aux/bbtime.h"
#include "BlackBox/Aux/bbselector.h"
#include "BlackBox/Aux/bbvector.h"
#include "BlackBox/Algorithms/bbsparse.h"

typedef BlackBoxURatFunc< SaclibQ >                         BBRatFuncSacQ;
typedef BlackBoxNumDen< SaclibQ, SaclibQX, BBRatFuncSacQ >  BBNumDen;
typedef BlackBoxSelector< SaclibQ, BBNumDen >               BBSelNumDenSacQ;
SaclibQ RFSacQ( SaclibQ* );

int main(  ) 
{
   Word stack;
   SaclibInitEnv(4000000,stack);
   int    i,N,Seed,Deg,DegNum,IsRestart;
   double Prob;
   MP_INT MPCard;
   char   ProbName[BBDEFS_FILEZ];

   mpz_init_set_ui( &MPCard, 100000007 ); 
   Deg       = -1;
   DegNum    = 2;
   Prob      = 1.0;
   Seed      = 1;
   N         = 2;
   IsRestart = BBDEFS_FALSE;
   sprintf( ProbName, "%d%s", N, "ratfuncsacQX" );
   GetCPUTime( ProbName );

   BBRatFuncSacQ   BBRFSacQ( RFSacQ, N, Deg, Prob );
   BBNumDen        NumDenRFSacQ( BBRFSacQ, &MPCard, DegNum, Prob, Seed );
   BBSelNumDenSacQ NumSacQ( NumDenRFSacQ, 0 );

   BBSPARSEAUX_VarVector Vars;
   BBSPARSEAUX_DegVector Degs;
   Vars.push_back(1); Vars.push_back(1);
   Degs.push_back(2); Degs.push_back(1);
   SaclibQX  SaclibQXElm[BBSPARSEAUX_MAXTERMS];
   BBSPARSEAUX_VecDegVector PruneDegs;
   bbvector< SaclibQ >      PruneMons;

   SparseInterp( NumSacQ, Vars, Degs, DegNum, &MPCard, 
                 PruneDegs, PruneMons, SaclibQXElm, ProbName, IsRestart );

   mpz_clear( &MPCard );
   SaclibCleanUpEnv();
   GetCPUTime( ProbName );
}

SaclibQ RFSacQ(
   SaclibQ* Pnts_i
   ) 
{
   SaclibQ X,Y;
   X = Pnts_i[0];
   Y = Pnts_i[1];
   return ( ((X*X) + Y) / (Y*Y) );
}

