/*-------------------------------------------------------------
  module: toe.C
  author: Angel L. Diaz
  date: 8/14/96

  Test the black box polynomial classes and its sub components.
-------------------------------------------------------------*/

// arithmetic includes
#include "PlugIns/saclib.h"
#include "BlackBox/CommonObjects/bbtoeplitz.h"


int main() {
   Word stack;
   SaclibInitEnv(1000000,stack);
   MP_INT Prime;
   mpz_init_set_ui( &Prime, 100003 );
   SaclibSetPrime( (MP_INT*)&Prime );

   // F := x^8 + x^6 - 3*x^4 - 3*x^3 + 8*x^2 + 2*x - 5;
   SaclibQX F = SaclibQX( (long)1, 8) + SaclibQX( (long)1, 6) -
                SaclibQX( (long)3, 4) - SaclibQX( (long)3, 3) +
                SaclibQX( (long)8, 2) + SaclibQX( (long)2, 1) - (long)5;
   // G := 3*x^6 + 5*x^4 - 4*x^2 - 9*x + 21;
   SaclibQX G = SaclibQX( (long)3, 6) + SaclibQX( (long)5, 4) -
                SaclibQX( (long)4, 2) - SaclibQX( (long)9, 1) + (long)21;    

   cout << "F = " << F << endl;
   cout << "G = " << G << endl;

   SaclibQ One;
   cout << "SubRes(F, G, 0) = " << SubRes( F, G, 0, One ) << endl;
   cout << "SubRes(F, G, 1) = " << SubRes( F, G, 1, One) << endl;
   cout << "SubRes(F, G, 2) = " << SubRes( F, G, 2, One) << endl;
   cout << "SubRes(F, G, 3) = " << SubRes( F, G, 3, One) << endl;
   cout << "SubRes(F, G, 4) = " << SubRes( F, G, 4, One) << endl;
   cout << "SubRes(F, G, 5) = " << SubRes( F, G, 5, One) << endl;
   cout << "SubRes(F, G, 6) = " << SubRes( F, G, 6, One) << endl;
   cout << "SubRes(F, G, 7) = " << SubRes( F, G, 7, One) << endl;
   cout << "SubRes(F, G, 8) = " << SubRes( F, G, 8, One) << endl;
   cout << "SubRes(F, G, 9) = " << SubRes( F, G, 9, One) << endl;

   // FF := x^8 + x^6 - 3*x^4 - 3*x^3 + 8*x^2 + 2*x - 5;
   SaclibZPX FF = SaclibZPX( (long)1, 8) + SaclibZPX( (long)1, 6) -
                SaclibZPX( (long)3, 4) - SaclibZPX( (long)3, 3) +
                SaclibZPX( (long)8, 2) + SaclibZPX( (long)2, 1) - (long)5;
   // GG := 3*x^6 + 5*x^4 - 4*x^2 - 9*x + 21;
   SaclibZPX GG = SaclibZPX( (long)3, 6) + SaclibZPX( (long)5, 4) -
                SaclibZPX( (long)4, 2) - SaclibZPX( (long)9, 1) + (long)21;    

   cout << "FF = " << FF << endl;
   cout << "GG = " << GG << endl;

   SaclibZP ZPOne;
   cout << "SubRes(FF, GG, 0) = " << SubRes( FF, GG, 0, ZPOne) << endl;
   cout << "SubRes(FF, GG, 1) = " << SubRes( FF, GG, 1, ZPOne) << endl;
   cout << "SubRes(FF, GG, 2) = " << SubRes( FF, GG, 2, ZPOne) << endl;
   cout << "SubRes(FF, GG, 3) = " << SubRes( FF, GG, 3, ZPOne) << endl;
   cout << "SubRes(FF, GG, 4) = " << SubRes( FF, GG, 4, ZPOne) << endl;
   cout << "SubRes(FF, GG, 5) = " << SubRes( FF, GG, 5, ZPOne) << endl;
   cout << "SubRes(FF, GG, 6) = " << SubRes( FF, GG, 6, ZPOne) << endl;
   cout << "SubRes(FF, GG, 7) = " << SubRes( FF, GG, 7, ZPOne) << endl;
   cout << "SubRes(FF, GG, 8) = " << SubRes( FF, GG, 8, ZPOne) << endl;
   cout << "SubRes(FF, GG, 9) = " << SubRes( FF, GG, 9, ZPOne) << endl;

   SaclibQ T1[4] = { (long)1, (long)2, (long)3, (long)4 };
   SaclibQX TO;
   cout << "ToeDet( T1, T1, 4, TO ) = " << ToeDet( T1, T1, 4, TO ) << endl;

   int NumVars = 4;
   int Deg     = -1;
   int Prob    = 1;
   BlackBoxSymToeDet< SaclibQ, SaclibQX > BBToeDet( NumVars, Deg, Prob );
   cout << "BBToeDet( T1 ) = " << *BBToeDet( T1 ) << endl;

   mpz_clear( &Prime );
   SaclibCleanUpEnv();
}


