#-----------------------------------------------------------------------
# file: examples.mpl
#
# author: Angel Diaz
# date: 11/25/96
#
# (c) Copyright 1996 Angel Diaz and Eirch Kaltofen, All Rights Reserved
# This code is provided WITHOUT WARRANTY either expressed or implied.
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
#
# description:
#    Construct a factor box for a 4x4 symmetric toeplitz matrix 
#    determinant. Evaluate all of the factors at a point. Both
#    factor box construction and evaluation occurs over the rationals.
#
#-----------------------------------------------------------------------

read `bridge.mpl`;
readlib(sinterp):
FoxBoxInitEnv( `bpid.bbs`, `cpid.bbs`, `command.bbs`, `ans.bbs` );

SymToeIndex := BlackBoxSymToe(BBNET_Q, 4, -1, 1.0);
FactorIndex := BlackBoxFactors(BBNET_Q, SymToeIndex, 100000007, 1.0, 103069);
Values      := BlackBoxEval(BBNET_Q, BBNET_FACS_EVAL, FactorIndex, 
               [10,20,30,40]);

FoxBoxCleanUp();

#-----------------------------------------------------------------------
#
# description: 
#    Interpolate both factors of a 4x4 symmetric toeplitz matrix determinant.
#    Two interpolation algorithms are tested
#      1) Local application of Maple's native sparse conversion
#      2) Remote application of FoxBox's sparse conversion
#    Both the factor box construction and evaluation occurs over the rationals.
#
#-----------------------------------------------------------------------

read `bridge.mpl`;
readlib(sinterp):
FoxBoxInitEnv( `bpid.bbs`, `cpid.bbs`, `command.bbs`, `ans.bbs` );

SymToeIndex := BlackBoxSymToe(BBNET_Q, 4, -1, 1.0);
FactorIndex := BlackBoxFactor(BBNET_Q, SymToeIndex, 0, 100000007, 1.0, 103069);
f := proc( x1, x2, x3, x4, p)
   local FacValue;
   FacValue    := BlackBoxEval(BBNET_Q, BBNET_FAC_EVAL, FactorIndex,
                  [x1, x2, x3,x4] );
   RETURN( FacValue mod p );
end:
F1 := sinterp( f, [x1, x2, x3, x4], 2, 100000007 );
FB1 := SparseConversion( BBNET_Q, FactorIndex, [x1, x2, x3, x4 ],
   [ 4, 4, 4, 4 ], 4, 100000007 );

FactorIndex := BlackBoxFactor(BBNET_Q, SymToeIndex, 1, 100000007, 1.0, 103069);
f := proc( x1, x2, x3, x4, p)
   local FacValue;
   FacValue    := BlackBoxEval(BBNET_Q, BBNET_FAC_EVAL, FactorIndex,
                  [x1, x2, x3,x4] );
   RETURN( FacValue mod p );
end:
F2 := sinterp( f, [x1, x2, x3, x4], 2, 100000007 );
FB2 := SparseConversion( BBNET_Q, FactorIndex, [x1, x2, x3, x4 ],
   [ 4, 4, 4, 4 ], 4, 100000007 );

FoxBoxCleanUp();

#-----------------------------------------------------------------------
#
# description:
#    Interpolate the first factor of a homomorphic map factor box of a
#    6x6 symmetric toeplitz matrix determinant factor box. Two interpolation 
#    algorithms are tested
#      1) Local application of Maple's native  sparse conversion
#      2) Remote application of FoxBox's sparse conversion
#
#-----------------------------------------------------------------------

read `bridge.mpl`;
readlib(sinterp):
FoxBoxInitEnv( `bpid.bbs`, `cpid.bbs`, `command.bbs`, `ans.bbs` );
FoxBoxSetPrime( 100000007 );

SymToeIndexQ  := BlackBoxSymToe(BBNET_Q, 6, -1, 1.0);
SymToeIndexZP := BlackBoxSymToe(BBNET_ZP, 6, -1, 1.0);
FactorsIndexQ := BlackBoxFactors(BBNET_Q,SymToeIndexQ,100000007,1.0,103069);
FactorsIndexZP:= BlackBoxHomomorphicMap(BBNET_FACS,FactorsIndexQ,SymToeIndexZP);
FactorIndexZP := BlackBoxSelectValue(BBNET_ZP,FactorsIndexZP,0);

f := proc( x1, x2, x3, x4, x5, x6, p)
   local FactorValue;
   FactorValue   := BlackBoxEval( BBNET_ZP, BBNET_FAC_HMAP_EVAL, 
                    FactorIndexZP, [x1,x2,x3,x4,x5,x6] );
   RETURN( FactorValue );
end:
F1 := sinterp( f, [x1, x2, x3, x4, x5, x6 ], 6, 100000007 );

FB2 := SparseConversion( BBNET_ZP, FactorIndexZP, [x1, x2, x3, x4, x5, x6 ],
   [ 6, 6, 6, 6, 6, 6 ], 6, 100000007 );

FoxBoxCleanUp();

#-----------------------------------------------------------------------
#
# description:
#    Interpolate the gcd of two Vandermonde matrix determinants which have
#    two variables in common. Two interpolation algorithms are tested
#      1) Local application of Maple's native  sparse conversion 
#      2) Remote application of FoxBox's sparse conversion 
#    Both the GCD box construction and evaluation occur over Z mod P.
#
#-----------------------------------------------------------------------

read `bridge.mpl`;
readlib(sinterp):
FoxBoxInitEnv( `bpid.bbs`, `cpid.bbs`, `command.bbs`, `ans.bbs` );
FoxBoxSetPrime( 100000007 );

Vand1Index  := BlackBoxVand(BBNET_ZP, BBNET_VAND, 24, -1, 1.0);
Vand2Index  := BlackBoxVand(BBNET_ZP, BBNET_VANDSHIFT, 24, -1, 1.0);
GCDIndex    := BlackBoxGCD(BBNET_ZP, [Vand1Index,Vand2Index], 100000007, 1.0, 
               103069);
f := proc( x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13, x14,
           x15, x16, x17, x18, x19, x20, x21, x22, x23, x24, p) 
   local GCDValue;
   GCDValue    := BlackBoxEval(BBNET_ZP, BBNET_GCD_EVAL, GCDIndex,
               [x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13, 
                x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24 ] );
   RETURN( GCDValue );
end:
sinterp( f, [x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13,
   x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24], 1, 100000007 );

SparseConversion( BBNET_ZP, GCDIndex, [x1, x2, x3, x4, x5, x6, x7, x8, x9, 
   x10, x11, x12, x13, x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24],
   [ 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
   1, 100000007 );

FoxBoxCleanUp(); 

#-----------------------------------------------------------------------
#
# description:
#    Interpolate the gcd of two Vandermonde matrix determinants which have
#    two variables in common. FoxBox's interpolation algorithm is tested
#    over Q.
#
#-----------------------------------------------------------------------

read `bridge.mpl`;
readlib(sinterp):
FoxBoxInitEnv( `bpid.bbs`, `cpid.bbs`, `command.bbs`, `ans.bbs` );

Vand1Index  := BlackBoxVand(BBNET_Q, BBNET_VAND, 6, -1, 1.0);
Vand2Index  := BlackBoxVand(BBNET_Q, BBNET_VANDSHIFT, 6, -1, 1.0);
GCDIndex    := BlackBoxGCD(BBNET_Q, [Vand1Index,Vand2Index], 100000007, 1.0,
               103069);

SparseConversion( BBNET_Q, GCDIndex, [x1, x2, x3, x4, x5, x6 ],
   [ 1, 1, 0, 0, 0, 0 ], 1, 100000007 );

FoxBoxCleanUp();


