
/*-------------------------------------------------------------
  module: bbserver.h
  author: Angel L. Diaz

  date: 11/11/96

  description: 

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include "bbnetwork.h"

#include "PlugIns/saclib.h"
#include "PlugIns/maps.h"
#include "BlackBox/Aux/bbdefs.h"
#include "BlackBox/Aux/bbselector.h"
#include "BlackBox/Aux/bbvector.h"
#include "BlackBox/Aux/bbtime.h"
#include "BlackBox/CommonObjects/bbvandermonde.h"
#include "BlackBox/CommonObjects/bbtoeplitz.h"
#include "BlackBox/Algorithms/bbgcd.h"
#include "BlackBox/Algorithms/bbfactors.h"
#include "BlackBox/HomomorphicMaps/bbfachmap.h"
#include "BlackBox/Algorithms/bbsparse.h"

#include "bbservertrace.h"

#define BBSERVER_MAXOBJ      4
#define BBSERVER_NUMOBJ      4
#define BBSERVER_MAXVARS     100

typedef BlackBox< SaclibQ >                                  BBSacQX;
typedef BlackBox< SaclibZP >                                 BBSacZPX;
typedef BlackBoxVandDet< SaclibZP >                          BBVandDetSacZPX;
typedef BlackBoxVandDet< SaclibQ >                           BBVandDetSacQX;
typedef BlackBoxSymToeDet< SaclibQ, SaclibQX >               BBSymToeDetSacQX;
typedef BlackBoxSymToeDet< SaclibZP, SaclibZPX >             BBSymToeDetSacZPX;
typedef BlackBoxGCD< SaclibZP, SaclibZPX, BBSacZPX >         BBGCDSacZPX;
typedef BlackBoxGCD< SaclibQ, SaclibQX, BBSacQX >            BBGCDSacQX;
typedef BlackBoxFactors< SaclibQ, SaclibQX, BBSacQX >        BBFactorsSacQX;
typedef BlackBoxFactors< SaclibZP, SaclibZPX, BBSacZPX >     BBFactorsSacZPX;
typedef BlackBoxSelector< SaclibQ, BBSacQX >                 BBSelSacQX;
typedef BlackBoxSelector< SaclibZP, BBSacZPX >               BBSelSacZPX;
typedef BlackBoxFactorsHMap< SaclibQ, SaclibQX, SaclibZP, 
           SaclibZPX, BBSacQX, BBSacZPX, SaclibQSaclibZP >   BBFactorsHMap;

/*-------------------------------------------------------------
  class: BlackBoxHouse
  date: 11/14/96

  description:

-------------------------------------------------------------*/

class BlackBoxHouse {
   private:
      BBVandDetSacZPX          VandDetSacZPX[BBSERVER_MAXOBJ];
      BBVandDetSacQX           VandDetSacQX[BBSERVER_MAXOBJ];
      BBGCDSacZPX              GCDSacZPX[BBSERVER_MAXOBJ];
      BBGCDSacQX               GCDSacQX[BBSERVER_MAXOBJ];
      BBSacZPX*                BBArrayZPX[BBSERVER_MAXOBJ*BBSERVER_NUMOBJ];
      BBSacQX*                 BBArrayQX[BBSERVER_MAXOBJ*BBSERVER_NUMOBJ];
      BBSymToeDetSacQX         SymToeDetSacQX[BBSERVER_MAXOBJ];
      BBSymToeDetSacZPX        SymToeDetSacZPX[BBSERVER_MAXOBJ];
      BBFactorsSacQX           FacsSacQX[BBSERVER_MAXOBJ];
      BBFactorsSacZPX          FacsSacZPX[BBSERVER_MAXOBJ];
      BBSelSacQX               FacSacQX[BBSERVER_MAXOBJ];
      BBSelSacZPX              FacSacZPX[BBSERVER_MAXOBJ];
      BBFactorsHMap            FacsHMap[BBSERVER_MAXOBJ];
      BBSelSacZPX              FacHMap[BBSERVER_MAXOBJ];
      SaclibQSaclibZP          h;
      struct IndexNode {
         int IndexType;
         int Index;
      }                        IndexArray[BBSERVER_MAXOBJ*BBSERVER_NUMOBJ];
      int IndexTop;
      int VDSacZPXTop;
      int VDSacQXTop;
      int GCDSacZPXTop;
      int GCDSacQXTop;
      int TDSacZPXTop;
      int TDSacQXTop;
      int FacsSacZPXTop;
      int FacsSacQXTop;
      int FacSacZPXTop;
      int FacSacQXTop;
      int FacsHMapTop;
      int FacHMapTop;
   public:   
      BlackBoxHouse( );
      ~BlackBoxHouse( );

      inline int ConsVandDet( int, int, int, double );
      inline int ConsVandDetShift( int, int, int, double );
      inline int ConsGCD( int, int*, int, MP_INT*, double, int );     
      inline int ConsSymToeDet( int, int, int, double );
      inline int ConsFac( int, int, int, MP_INT*, double, int );    
      inline int ConsFacs( int, int, MP_INT*, double, int ); 
      inline int ConsFacsHMap( int, int );
      inline int ConsSel( int, int, int );
      inline SaclibZP  EvalZP( int, int, SaclibZP*, int& );
      inline SaclibQ   EvalQ( int, int, SaclibQ*, int& );
      inline SaclibQ*  FacsEvalQ( int, int, SaclibQ*, int&, int& );
      inline SaclibZP* FacsHMapEval( int, int, SaclibZP*, int&, int& );
      inline void SparseConversionZP( int, int, int, BBSPARSEAUX_VarVector&, 
         BBSPARSEAUX_DegVector&, int, MP_INT*, SaclibZPX*, 
         BBSPARSEAUX_VecDegVector&, bbvector< SaclibZP >&, int& );
      inline void SparseConversionQ( int, int, int, BBSPARSEAUX_VarVector&, 
         BBSPARSEAUX_DegVector&, int, MP_INT*, SaclibQX*, 
         BBSPARSEAUX_VecDegVector&, bbvector< SaclibQ >&, int& );
      inline BBSacZPX* GetHandleZPX( int, int, int&, int& );
      inline BBSacQX*  GetHandleQX( int, int, int&, int& );
};

BlackBoxHouse::BlackBoxHouse( ) { 
   IndexTop      = 0;
   VDSacZPXTop   = 0;
   VDSacQXTop    = 0;
   GCDSacZPXTop  = 0;
   GCDSacQXTop   = 0;
   TDSacZPXTop   = 0;
   TDSacQXTop    = 0;
   FacsSacZPXTop = 0;
   FacsSacQXTop  = 0;
   FacsHMapTop   = 0;
   FacSacZPXTop  = 0;
   FacSacQXTop   = 0;
}

BlackBoxHouse::~BlackBoxHouse( ) { 
}

inline int BlackBoxHouse::ConsVandDet(
   int    Field_i,
   int    N_i,
   int    DegDet_i,
   double Prob_i
   )
{
   int Index = BBNET_ERROR;

   if ( Field_i == BBNET_ZP ) {
      if ( VDSacZPXTop >= BBSERVER_MAXOBJ )
         goto XDONE;
      Index                       = IndexTop;
      IndexArray[Index].Index     = VDSacZPXTop;
      IndexArray[Index].IndexType = BBNET_VAND;
      VandDetSacZPX[VDSacZPXTop].Construct( N_i, DegDet_i, Prob_i, N_i/2 ); 
      VDSacZPXTop++; IndexTop++; 
   }
   else {
      if ( VDSacQXTop >= BBSERVER_MAXOBJ )
         goto XDONE;
      Index                       = IndexTop;
      IndexArray[Index].Index     = VDSacQXTop;
      IndexArray[Index].IndexType = BBNET_VAND;
      VandDetSacQX[VDSacQXTop].Construct( N_i, DegDet_i, Prob_i, N_i/2 );
      VDSacQXTop++; IndexTop++;
   }

   XDONE:
   return( Index );
}

inline int BlackBoxHouse::ConsVandDetShift(
   int    Field_i,
   int    N_i,
   int    DegDet_i,
   double Prob_i
   )
{
   int Index = BBNET_ERROR;

   if ( Field_i == BBNET_ZP ) {
      if ( VDSacZPXTop >= BBSERVER_MAXOBJ )
         goto XDONE;
      Index                       = IndexTop;
      IndexArray[Index].Index     = VDSacZPXTop;
      IndexArray[Index].IndexType = BBNET_VANDSHIFT;
      VandDetSacZPX[VDSacZPXTop].Construct( N_i, DegDet_i, Prob_i, N_i/2,
         N_i/2, N_i, BBDEFS_TRUE ); 
      VDSacZPXTop++; IndexTop++; 
   }
   else {
      if ( VDSacQXTop >= BBSERVER_MAXOBJ )
         goto XDONE;
      Index                       = IndexTop;
      IndexArray[Index].Index     = VDSacQXTop;
      IndexArray[Index].IndexType = BBNET_VAND;
      VandDetSacQX[VDSacQXTop].Construct( N_i, DegDet_i, Prob_i, N_i/2,
         N_i/2, N_i, BBDEFS_TRUE );
      VDSacQXTop++; IndexTop++;
   }

   XDONE:
   return( Index );
}

inline int BlackBoxHouse::ConsSymToeDet(
   int    Field_i,
   int    N_i,
   int    DegDet_i,
   double Prob_i
   )
{
   int Index = BBNET_ERROR;

   if ( Field_i == BBNET_ZP ) {
      if ( TDSacZPXTop >= BBSERVER_MAXOBJ )
         goto XDONE;
      Index                       = IndexTop;
      IndexArray[Index].Index     = TDSacZPXTop;
      IndexArray[Index].IndexType = BBNET_TOE;
      SymToeDetSacZPX[TDSacZPXTop].Construct( N_i, DegDet_i, Prob_i ); 
      TDSacZPXTop++; IndexTop++; 
   }
   else {
      if ( TDSacQXTop >= BBSERVER_MAXOBJ )
         goto XDONE;
      Index                       = IndexTop;
      IndexArray[Index].Index     = TDSacQXTop;
      IndexArray[Index].IndexType = BBNET_TOE;
      SymToeDetSacQX[TDSacQXTop].Construct( N_i, DegDet_i, Prob_i );
      TDSacQXTop++; IndexTop++;
   }

   XDONE:
   return( Index );
}

inline BBSacZPX* BlackBoxHouse::GetHandleZPX( 
   int  Index_i, 
   int  IndexType_i,
   int& Status_m,
   int& RetVal_m
   )
{
   Status_m = BBNET_ERROR;

   switch( IndexType_i ) {
         case BBNET_TOE:
            if ( Index_i > TDSacZPXTop )
               goto XDONE;       
            Status_m = BBNET_SUCCESS;
            return( &SymToeDetSacZPX[Index_i] );
         break;
         case BBNET_VAND:
            if ( Index_i > VDSacZPXTop )
               goto XDONE;       
            Status_m = BBNET_SUCCESS;
            return( &VandDetSacZPX[Index_i] );
         break;
         case BBNET_VANDSHIFT:
            if ( Index_i > VDSacZPXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            return( &VandDetSacZPX[Index_i] );
         break;
         case BBNET_FAC:
            if ( Index_i > FacsSacZPXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            return( &FacsSacZPX[Index_i] );
         break;
         case BBNET_FAC_EVAL:
            if ( Index_i > FacSacZPXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            return( &FacSacZPX[Index_i] );
         break;
         case BBNET_FACS_EVAL:
            if ( Index_i > FacsSacZPXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            RetVal_m = FacsSacZPX[Index_i].GetNumFactors();
            return( &FacsSacZPX[Index_i] );
         break;
         case BBNET_FACS_HMAP_EVAL:
            if ( Index_i > FacsHMapTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            RetVal_m = FacsHMap[Index_i].GetNumFactors();
            return( &FacsHMap[Index_i] );
         break;
         case BBNET_FAC_HMAP_EVAL:
            if ( Index_i > FacHMapTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            return( &FacHMap[Index_i] );
         break;
         case BBNET_GCD:
            if ( Index_i > GCDSacZPXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            return( &GCDSacZPX[Index_i] );
         break;
         case BBNET_GCD_EVAL:
            if ( Index_i > GCDSacZPXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            return( &GCDSacZPX[Index_i] );
         break;
   }
   XDONE:

#ifdef BBDEFS_SERVER_TRC
BBHouseGetHandleZPXTrace1( Index_i, IndexType_i );
#endif

   return( (BBSacZPX*) NULL );
}

inline BBSacQX*  BlackBoxHouse::GetHandleQX( 
   int  Index_i,
   int  IndexType_i,
   int& Status_m,
   int& RetVal_m
   )
{
   Status_m = BBNET_ERROR;

   switch( IndexType_i ) {
         case BBNET_TOE:
            if ( Index_i > TDSacQXTop )
               goto XDONE;       
            Status_m = BBNET_SUCCESS;
            return( &SymToeDetSacQX[Index_i] );
         break;
         case BBNET_VAND:
            if ( Index_i > VDSacQXTop )
               goto XDONE;       
            Status_m = BBNET_SUCCESS;
            return( &VandDetSacQX[Index_i] );
         break;
         case BBNET_VANDSHIFT:
            if ( Index_i > VDSacQXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            return( &VandDetSacQX[Index_i] );
         break;
         case BBNET_FAC:
            if ( Index_i > FacsSacQXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            return( &FacsSacQX[Index_i] );
         break;
         case BBNET_FAC_EVAL:
            if ( Index_i > FacSacQXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            return( &FacSacQX[Index_i] );
         break;
         case BBNET_FACS_EVAL:
            if ( Index_i > FacsSacQXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            RetVal_m = FacsSacQX[Index_i].GetNumFactors();
            return( &FacsSacQX[Index_i] );
         break;
         case BBNET_GCD:
            if ( Index_i > GCDSacQXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            return( &GCDSacQX[Index_i] );
         break;
         case BBNET_GCD_EVAL:
            if ( Index_i > GCDSacQXTop )
               goto XDONE;
            Status_m = BBNET_SUCCESS;
            return( &GCDSacQX[Index_i] );
         break;
   }
   XDONE:

#ifdef BBDEFS_SERVER_TRC
BBHouseGetHandleQXTrace1( Index_i, IndexType_i );
#endif

   return( (BBSacQX*) NULL );
}

inline int BlackBoxHouse::ConsGCD(
   int     r_i,
   int*    IndexArray_i,
   int     Field_i,
   MP_INT* Card_i,
   double  Prob_i,
   int     Seed_i
   )
{
   int i,status,rval;
   int Index = BBNET_ERROR;

   if ( Field_i == BBNET_ZP ) {
      if ( GCDSacZPXTop >= BBSERVER_MAXOBJ )
         goto XDONE;
      // populate array of black boxes
      for( i=0; i<r_i; i++ ) {
         BBArrayZPX[i] = GetHandleZPX( IndexArray[IndexArray_i[i]].Index,
            IndexArray[IndexArray_i[i]].IndexType, status, rval );
         if ( status != BBNET_SUCCESS )
            goto XDONE;
      }
      // construct black box GCD over ZP
      Index                       = IndexTop;
      IndexArray[Index].Index     = GCDSacZPXTop;
      IndexArray[Index].IndexType = BBNET_GCD;
      GCDSacZPX[GCDSacZPXTop].Construct(BBArrayZPX, r_i, Card_i, Prob_i, 
         Seed_i); 
      GCDSacZPXTop++; IndexTop++; 
   }
   else {
      if ( GCDSacQXTop >= BBSERVER_MAXOBJ )
         goto XDONE;
      // populate array of black boxes
      for( i=0; i<r_i; i++ ) {
         BBArrayQX[i] = GetHandleQX( IndexArray[IndexArray_i[i]].Index,
            IndexArray[IndexArray_i[i]].IndexType, status, rval );
         if ( status != BBNET_SUCCESS )
            goto XDONE;
      }
      // construct black box GCD over Q
      Index                       = IndexTop;
      IndexArray[Index].Index     = GCDSacQXTop;
      IndexArray[Index].IndexType = BBNET_GCD;
      GCDSacQX[GCDSacQXTop].Construct(BBArrayQX, r_i, Card_i, Prob_i, Seed_i); 
      GCDSacQXTop++; IndexTop++; 
   }

   XDONE:
   return( Index );
}

inline SaclibZP BlackBoxHouse::EvalZP( 
   int       Index_i, 
   int       NumPnts_i, 
   SaclibZP* Pnts_i,
   int&      Status_m
   )
{
   int rval;
   MP_INT MPZero;
   SaclibZP SacZPValue;
   BBSacZPX *BB;

   mpz_init_set_si( &MPZero, 0 );
   SacZPValue = SaclibZP( &MPZero );

   BB         = GetHandleZPX( IndexArray[Index_i].Index, 
                   IndexArray[Index_i].IndexType, Status_m, rval );
   if ( Status_m == BBNET_SUCCESS ) {
      SacZPValue = *((*BB)( Pnts_i ));       
   }

   mpz_clear( &MPZero );
   return( SacZPValue );
}

inline SaclibQ  BlackBoxHouse::EvalQ( 
   int Index_i,
   int NumPnts_i,
   SaclibQ* Pnts_i,
   int& Status_m
   )
{
   int rval;
   MP_INT MPZero;
   SaclibQ SacQValue;
   BBSacQX *BB;

   mpz_init_set_si( &MPZero, 0 );
   SacQValue = SaclibQ( &MPZero );
   BB        = GetHandleQX( IndexArray[Index_i].Index, 
                   IndexArray[Index_i].IndexType, Status_m, rval );
   if ( Status_m == BBNET_SUCCESS ) {
      SacQValue = *((*BB)( Pnts_i ));       
   }

   mpz_clear( &MPZero );
   return( SacQValue );
}

inline SaclibQ*  BlackBoxHouse::FacsEvalQ(
   int Index_i,
   int NumPnts_i,
   SaclibQ* Pnts_i,
   int& NumFacs_m,
   int& Status_m
   )
{
   SaclibQ* SacQValue = NULL;
   BBSacQX *BB;

   BB        = GetHandleQX( IndexArray[Index_i].Index, 
                   IndexArray[Index_i].IndexType, Status_m, NumFacs_m );
   if ( Status_m == BBNET_SUCCESS ) {
      SacQValue = (*BB)( Pnts_i);       
   }

   return( SacQValue );
}

inline SaclibZP*  BlackBoxHouse::FacsHMapEval(
   int Index_i,
   int NumPnts_i,
   SaclibZP* Pnts_i,
   int& NumFacs_m,
   int& Status_m
   )
{
   SaclibZP* SacZPValue = NULL;
   BBSacZPX *BB;

   BB        = GetHandleZPX( IndexArray[Index_i].Index, 
                   IndexArray[Index_i].IndexType, Status_m, NumFacs_m );
   if ( Status_m == BBNET_SUCCESS ) {
      SacZPValue = (*BB)( Pnts_i);       
   }

   return( SacZPValue );
}

inline void BlackBoxHouse::SparseConversionZP(
   int Field_i,
   int Index_i,
   int NumVars_i,
   BBSPARSEAUX_VarVector& Vars_i,
   BBSPARSEAUX_DegVector& Degs_i,
   int TotalDeg_i,
   MP_INT* Card_i,
   SaclibZPX* SaclibZPXElm_i,
   BBSPARSEAUX_VecDegVector& PruneDegs_i,
   bbvector< SaclibZP >&     PruneMons_i,
   int& Status_m
   ) 
{
   int      rval;
   int      IsRestart = BBDEFS_FALSE;
   BBSacZPX *BBZPX;
   char*    ProbName = NULL;

   BBZPX = GetHandleZPX( IndexArray[Index_i].Index, 
             IndexArray[Index_i].IndexType, Status_m, rval );
   if ( Status_m != BBNET_SUCCESS )
      goto XDONE;

   SparseInterp( (*BBZPX), Vars_i, Degs_i, TotalDeg_i, Card_i, 
                  PruneDegs_i, PruneMons_i, SaclibZPXElm_i, ProbName,
                  IsRestart );

   XDONE:
   return;
}

inline void BlackBoxHouse::SparseConversionQ(
   int Field_i,
   int Index_i,
   int NumVars_i,
   BBSPARSEAUX_VarVector& Vars_i,
   BBSPARSEAUX_DegVector& Degs_i,
   int TotalDeg_i,
   MP_INT* Card_i,
   SaclibQX* SaclibQXElm_i,
   BBSPARSEAUX_VecDegVector& PruneDegs_i,
   bbvector< SaclibQ >&      PruneMons_i,
   int& Status_m
   ) 
{
   int     rval;
   int     IsRestart = BBDEFS_FALSE;
   BBSacQX *BBQX;
   char*   ProbName = NULL;

   BBQX = GetHandleQX( IndexArray[Index_i].Index, 
             IndexArray[Index_i].IndexType, Status_m, rval );
   if ( Status_m != BBNET_SUCCESS )
      goto XDONE;

   SparseInterp( (*BBQX), Vars_i, Degs_i, TotalDeg_i, Card_i, 
                  PruneDegs_i, PruneMons_i, SaclibQXElm_i, 
                  ProbName, IsRestart );

   XDONE:
   return;
}

inline int BlackBoxHouse::ConsFac(
   int     BoxIndex_i,
   int     FacIndex_i,
   int     Field_i,
   MP_INT* Card_i,
   double  Prob_i,
   int     Seed_i
   )
{
   int      i,status,rval;
   int      Index = BBNET_ERROR;
   BBSacZPX *BBZPX;
   BBSacQX  *BBQX; 

   if ( Field_i == BBNET_ZP ) {
       if ( FacSacZPXTop >= BBSERVER_MAXOBJ )
          goto XDONE;
       // MUST ADD ZP SUPPORT
    }
    else {
       if ( FacSacQXTop >= BBSERVER_MAXOBJ )
          goto XDONE;
       // construct black box factors over QX
       BBQX = GetHandleQX( IndexArray[BoxIndex_i].Index, 
                 IndexArray[BoxIndex_i].IndexType, status, rval );
       if ( status != BBNET_SUCCESS )
          goto XDONE;
       Index                       = IndexTop;
       IndexArray[Index].Index     = FacsSacQXTop;
       IndexArray[Index].IndexType = BBNET_FAC;
       FacsSacQX[FacsSacQXTop].Construct( *BBQX, Prob_i, Seed_i, Card_i ); 
       BBQX = &FacsSacQX[FacsSacQXTop];
       FacsSacQXTop++; IndexTop++;
       // select a factor
       Index                       = IndexTop;
       IndexArray[Index].Index     = FacSacQXTop;
       IndexArray[Index].IndexType = BBNET_FAC_EVAL;
       FacSacQX[FacSacQXTop].Construct( *BBQX, FacIndex_i ); 
       FacSacQXTop++; IndexTop++;
    }

    XDONE:
    return( Index );
}

inline int BlackBoxHouse::ConsFacs(
   int     BoxIndex_i,
   int     Field_i,
   MP_INT* Card_i,
   double  Prob_i,
   int     Seed_i
   )
{
   int      i,status,rval;
   int      Index = BBNET_ERROR;
   BBSacZPX *BBZPX;
   BBSacQX  *BBQX; 

   if ( Field_i == BBNET_ZP ) {
       if ( FacSacZPXTop >= BBSERVER_MAXOBJ )
          goto XDONE;
       // MUST ADD ZP SUPPORT
    }
    else {
       if ( FacSacQXTop >= BBSERVER_MAXOBJ )
          goto XDONE;
       // construct black box factors over QX
       BBQX = GetHandleQX( IndexArray[BoxIndex_i].Index, 
                 IndexArray[BoxIndex_i].IndexType, status, rval );
       if ( status != BBNET_SUCCESS )
          goto XDONE;
       Index                       = IndexTop;
       IndexArray[Index].Index     = FacsSacQXTop;
       IndexArray[Index].IndexType = BBNET_FACS_EVAL;
       FacsSacQX[FacsSacQXTop].Construct( *BBQX, Prob_i, Seed_i, Card_i ); 
       BBQX = &FacsSacQX[FacsSacQXTop];
       FacsSacQXTop++; IndexTop++;
    }

    XDONE:
    return( Index );
}

inline int BlackBoxHouse::ConsFacsHMap(
   int     BoxQIndex_i,
   int     BoxZPIndex_i
   ) 
{
   int      status,rval;
   int      Index = BBNET_ERROR;
   BBSacZPX *BBZPX;

   BBZPX = GetHandleZPX( IndexArray[BoxZPIndex_i].Index, 
             IndexArray[BoxZPIndex_i].IndexType, status, rval );
   if ( status != BBNET_SUCCESS )
      goto XDONE;

   Index                       = IndexTop;
   IndexArray[Index].Index     = FacsHMapTop;
   IndexArray[Index].IndexType = BBNET_FACS_HMAP_EVAL;
   // MUST CHANGE TO USE GETHANDLEQ
   FacsHMap[FacsHMapTop].Construct( *BBZPX, 
      FacsSacQX[IndexArray[BoxQIndex_i].Index], h ); 
   FacsHMapTop++; IndexTop++;

   XDONE:
   return( Index );

}

inline int BlackBoxHouse::ConsSel(
   int Field_i,
   int BoxIndex_i,
   int FacIndex_i
   ) 
{
   int      i,status,rval;
   int      Index = BBNET_ERROR, BoxIndexType;
   BBSacZPX *BBZPX;

   BoxIndexType = IndexArray[BoxIndex_i].IndexType;

   if ( Field_i == BBNET_ZP ) {
       if ( FacSacZPXTop >= BBSERVER_MAXOBJ )
          goto XDONE;
       BBZPX = GetHandleZPX( IndexArray[BoxIndex_i].Index, BoxIndexType, 
          status, rval );
       if ( status != BBNET_SUCCESS )
          goto XDONE;

       if ( BoxIndexType == BBNET_FACS_HMAP_EVAL ) {
          // select a factor
          Index                       = IndexTop;
          IndexArray[Index].Index     = FacHMapTop;
          IndexArray[Index].IndexType = BBNET_FACS_HMAP_EVAL;
          FacHMap[FacHMapTop].Construct( *BBZPX, FacIndex_i ); 
          FacHMapTop++; IndexTop++;
       }
       else {
          // MUST ADD SUPPORT FOR OTHER BOXES
       }
    }
    else {
       // MUST ADD Q SUPPORT
    }

   XDONE:
   return( Index );
}

