
/*-------------------------------------------------------------
  program: client.C
  author: Angel L. Diaz

  date: 11/11/96

  description: 

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#include <iostream.h>        // C++ input/output
#include <fstream.h>         // C++ file input/output
#include <signal.h>
#include "bbnetwork.h"
#include "BlackBox/Aux/bbdefs.h"
#include "clienttrace.h"

#define CLIENT_MAXFILEZ  100

int IsActive;

void HandleVand( int, ifstream&, ofstream&, char* );
void HandleGCD( int, ifstream&, ofstream&, char* );
void HandleGCDEval( int, ifstream&, ofstream&, char* );
void HandleSetPrime( int, ifstream&, char* );
void HandleToe( int, ifstream&, ofstream&, char* );
void HandleFac( int, ifstream&, ofstream&, char* );
void HandleFacEval( int, ifstream&, ofstream&, char* );
void HandleFacs( int, ifstream&, ofstream&, char* );
void HandleFacsEval( int, ifstream&, ofstream&, char* );
void HandleSel( int, ifstream&, ofstream&, char* );
void HandleFacsHMap( int, ifstream&, ofstream&, char* );
void HandleFacsHMapEval( int, ifstream&, ofstream&, char* );
void HandleFacHMapEval( int, ifstream&, ofstream&, char* );
void HandleSparseConversion( int, ifstream&, ofstream&, char* );

void ClientSetup( char * );
void ClientSignalHandle( int ); 

int main( int argc, char** argv ) 
{
   int  sock, n, i, Request,BPid;
   char mess[BBNET_MAXMESSZ];

   if ( argc != 5 ) {
      cerr << "client : usage: client <bridge_pid_file> <client_pid_file> "
           << "<command_file> <ans_file>" << endl;
      exit(-1);
   }

   // establish a stream connection with the FoxBox server 
   sock = Socket();
   ClientSetup( argv[2] );

   while ( BBDEFS_TRUE ) {

      IsActive = BBDEFS_FALSE;
      signal( SIGUSR2, ClientSignalHandle );
      while ( !(IsActive) ) { 
         sleep(1); 
      }

      ifstream BPidFile( argv[1] , ios::in );
      if ( !BPidFile ) {
         cerr << "client : cannot open " << argv[1]
              << " for input" << endl;
      }
      BPidFile >> BPid; 

      ifstream ComFile( argv[3] , ios::in );
      if ( !ComFile ) {
         cerr << "client : cannot open " << argv[3]
              << " for input" << endl;
      }

      ofstream AnsFile( argv[4], ios::out );
      if ( !AnsFile ) {
         cerr << "client : cannot open " << argv[4]
              << " for output" << endl;
      }

      // read/write request
      ComFile >> mess;
      Request   = atoi( mess );
      n         = strlen( mess );
      mess[n]   = '\n'; n++;
      if ( WriteLine( sock, mess, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );

      switch( Request ) {

         case BBNET_TOE:
            HandleToe( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_VAND:
            HandleVand( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_VANDSHIFT:
            HandleVand( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_FAC:
            HandleFac( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_FAC_EVAL:
            HandleFacEval( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_FACS:
            HandleFacs( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_FACS_EVAL:
            HandleFacsEval( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_SEL:
            HandleSel( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_FACS_HMAP:
            HandleFacsHMap( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_FACS_HMAP_EVAL:
            HandleFacsHMapEval( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_FAC_HMAP_EVAL:
            HandleFacHMapEval( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_GCD:
            HandleGCD( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_GCD_EVAL:
            HandleGCDEval( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_SPARSE:
            HandleSparseConversion( sock, ComFile, AnsFile, mess );
         break;

         case BBNET_SETPRIME:
            HandleSetPrime( sock, ComFile,  mess );
         break;

         case BBNET_DELETE:
         break;

         case BBNET_KILL:
            ComFile.close();
            AnsFile.close();
            BPidFile.close();
            kill( BPid, SIGUSR2 );
            goto XDONE;
         break;

      }

      ComFile.close();
      AnsFile.close();
      BPidFile.close();
      kill( BPid, SIGUSR2 );
   } 

   XDONE:
   close(sock);
   exit(0);
}

void HandleSetPrime( 
   int       sock_i,
   ifstream& ComFile_i,
   char*     mess_m
   )
{
   int  n;
   // prime
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
       ErrorExit( "Client : WriteLine error", errno );

#ifdef BBDEFS_SERVER_TRC
BBCLIENTSetPrimeTrace1( mess_m );
#endif

}

void HandleVand( 
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int i, n;
   // field, number of variables, degreee, probability
   for( i=0; i<4; i++ ) {
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 
   // recieve index
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTVandTrace1( mess_m );
#endif

}

void HandleGCD( 
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int i, n, r;
   // numeber of BB's
   ComFile_i >> mess_m;
   r         = atoi( mess_m );
   n         = strlen( mess_m );
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );
   // index of each BB
   for( i=0; i<r; i++) {      
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 
   // field, card, probability, seed 
   for( i=0; i<4; i++ ) {
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 
   // recieve index
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTGCDTrace1( mess_m );
#endif

}

void HandleGCDEval( 
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int i, n, NumPnts;

   // field
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // index
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // numer of points
   ComFile_i >> mess_m;
   NumPnts   = atoi( mess_m );
   n         = strlen( mess_m );
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // each point
   for( i=0; i<NumPnts; i++) {      
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 

   // recieve value
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTGCDEvalTrace1( mess_m );
#endif

}

void HandleToe( 
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int i, n;
   // field, number of variables, degreee, probability
   for( i=0; i<4; i++ ) {
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 
   // recieve index
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTToeTrace1( mess_m );
#endif

}

void HandleFac( 
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int i, n;
   // box index, factor index, field, card, probability, seed 
   for( i=0; i<6; i++ ) {
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 
   // recieve index
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTFacTrace1( mess_m );
#endif

}

void HandleFacEval(
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int i, n, NumPnts;

   // field
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // index
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // numer of points
   ComFile_i >> mess_m;
   NumPnts   = atoi( mess_m );
   n         = strlen( mess_m );
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // each point
   for( i=0; i<NumPnts; i++) {      
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 

   // recieve value
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTFacEvalTrace1( mess_m );
#endif

}

void HandleFacs( 
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int i, n;
   // box index, field, card, probability, seed 
   for( i=0; i<5; i++ ) {
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 
   // recieve index
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTFacsTrace1( mess_m );
#endif

}

void HandleFacsEval(
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int i, n, NumPnts,NumFacs;

   // field
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // index
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // numer of points
   ComFile_i >> mess_m;
   NumPnts   = atoi( mess_m );
   n         = strlen( mess_m );
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // each point
   for( i=0; i<NumPnts; i++) {      
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 

   // recieve number of factors
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   NumFacs = atoi( mess_m );
   AnsFile_i << mess_m;

   // recieve values
   for( i=0; i<NumFacs; i++) {
      n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
      CheckReadStatus( n ); 
      AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTFacsEvalTrace1( mess_m );
#endif

   }
}

void HandleFacsHMapEval(
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int i, n, NumPnts,NumFacs;

   // index
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // numer of points
   ComFile_i >> mess_m;
   NumPnts   = atoi( mess_m );
   n         = strlen( mess_m );
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // each point
   for( i=0; i<NumPnts; i++) {      
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 

   // recieve number of factors
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   NumFacs = atoi( mess_m );
   AnsFile_i << mess_m;

   // recieve values
   for( i=0; i<NumFacs; i++) {
      n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
      CheckReadStatus( n ); 
      AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTFacsHMapEvalTrace1( mess_m );
#endif

   }
}

void HandleFacHMapEval(
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int i, n, NumPnts,NumFacs;

   // index
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // numer of points
   ComFile_i >> mess_m;
   NumPnts   = atoi( mess_m );
   n         = strlen( mess_m );
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // each point
   for( i=0; i<NumPnts; i++) {      
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 

   // recieve value
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTFacHMapEvalTrace1( mess_m );
#endif

}

void HandleSel(
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int n;

   // field
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
       ErrorExit( "Client : WriteLine error", errno );

   // index of multiple value black box 
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
       ErrorExit( "Client : WriteLine error", errno );

   // selected value
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
       ErrorExit( "Client : WriteLine error", errno );

   // recieve index
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTSelTrace1( mess_m );
#endif

}

void HandleFacsHMap(
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int n;
   // index of factor box over Q
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
       ErrorExit( "Client : WriteLine error", errno );

   // index of original box over ZP
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
       ErrorExit( "Client : WriteLine error", errno );

   // recieve index
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   AnsFile_i << mess_m;

#ifdef BBDEFS_SERVER_TRC
BBCLIENTHMapTrace1( mess_m );
#endif

}

void HandleSparseConversion(
   int       sock_i,
   ifstream& ComFile_i,
   ofstream& AnsFile_i,
   char*     mess_m
   )
{
   int i,j, n, NumVars, NumMons, NumDegs;

   // field
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // index
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // number of variables
   ComFile_i >> mess_m;
   NumVars   = atoi( mess_m );
   n         = strlen( mess_m );
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // degree bound for each variable
   for( i=0; i<NumVars; i++) {      
      ComFile_i >> mess_m;
      n = strlen(mess_m);
      mess_m[n] = '\n'; n++;
      if ( WriteLine( sock_i, mess_m, n )  != n )
          ErrorExit( "Client : WriteLine error", errno );
   } 

   // bound on total degree
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // cardinality a set from which to choose random elements
   ComFile_i >> mess_m;
   n = strlen(mess_m);
   mess_m[n] = '\n'; n++;
   if ( WriteLine( sock_i, mess_m, n )  != n )
      ErrorExit( "Client : WriteLine error", errno );

   // recieve number of monomials
   n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
   CheckReadStatus( n ); 
   NumMons = atoi( mess_m );
   AnsFile_i << mess_m;

   for( i=0; i<NumMons; i++ ) {

      // ith monomial
      n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
      CheckReadStatus( n ); 
      AnsFile_i << mess_m;

      // number of degs 
      n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
      CheckReadStatus( n ); 
      NumDegs = atoi( mess_m );
      AnsFile_i << mess_m;

      // each degree
      for( j=0; j<NumDegs; j++ ) {
         n = ReadLine( sock_i, mess_m, BBNET_MAXMESSZ );
         CheckReadStatus( n ); 
         AnsFile_i << mess_m;
      } 

   }

#ifdef BBDEFS_SERVER_TRC
BBCLIENTSCTrace1( );
#endif

}

void ClientSetup(
   char *PidFileName
   )
{
   int MyPid;

   ofstream CPidFile( PidFileName , ios::out );
   if ( !CPidFile ) {
      cerr << "client : cannot open " << PidFileName
           << " for input" << endl;
   }
 
   MyPid = getpid();

   CPidFile << MyPid;
   CPidFile.close();
}

void ClientSignalHandle ( int ) {
   IsActive = BBDEFS_TRUE;
}

