
/*-------------------------------------------------------------
  module: bbnumdentrace.h
  author: Angel L. Diaz

  date: 11/1/96

  desciption: 
     Tracing routines for: bbnumden.h
 
     PRINTLEVEL = 0 ; tracing off
     PRINTLEVEL > 0 ; function tags
     PRINTLEVEL > 1 ; milestone messages
     PRINTLEVEL > 2 ; state variables
     PRINTLEVEL > 3 ; state variables and arrays/vectors

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.

-------------------------------------------------------------*/

#ifndef _BBNUMDENTRACE
#define _BBNUMDENTRACE

#ifdef BBDEFS_NUMDEN_TRC

#include "BlackBox/Aux/bbtrace.h"

template< class K, class P > 
void BBNUMDENTrace1(
   double       Prob_i,
   unsigned int Seed_i,
   unsigned int NumVariables_i,
   K&           GenElm_i,
   int          StaticDeg_i,
   K*           RandA_i,
   K*           RandB_i,
   int          NumAB_i,
   P*           LinForms_i,
   int          NumLinForms_i 
   )
{
   if ( PRINTLEVEL > 0 ) {
   cout << "bbnumden : Constructing black box numerator/denominator" << endl;
   }
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("bbnumden : ","Prob = ",Prob_i); 
   OutputSingleItem("bbnumden : ","Seed = ",Seed_i);
   OutputSingleItem("bbnumden : ","NumVariables = ",NumVariables_i);
   OutputSingleItem("bbnumden : ","GenElm = ",GenElm_i);
   OutputSingleItem("bbnumden : ","StaticDeg = ",StaticDeg_i);
   }
   if ( PRINTLEVEL > 3 ) {
   OutputArray("bbnumden : ","Random elements a2, ... , an",
               "RandA[",RandA_i,0,NumAB_i);
   OutputArray("bbnumden : ","Random elements b2, ... , bn",
               "RandB[",RandB_i,0,NumAB_i);
   OutputArray("bbnumden : ","Linear Forms",
               "LinForms[",LinForms_i,0,NumLinForms_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbnumden : Probabilistically computing degrees of f and g" << endl;
   }
}

template< class K, class P > 
void BBNUMDENTrace2(
   P&           FHat_i,
   P&           GHat_i,
   K*           FHatCoeffs_i,
   unsigned int D_i,
   unsigned int E_i,
   K&           LC_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputSingleItem("bbnumden : ","FHat =",FHat_i);
   OutputSingleItem("bbnumden : ","Degree of f = ", D_i);
   OutputSingleItem("bbnumden : ","GHat =",GHat_i);
   OutputSingleItem("bbnumden : ","Degree of g = ", E_i);
   OutputArray("bbnumden : ","Coefficients of FHat",
               "FHatCoeffs[",FHatCoeffs_i,0,D_i);
   OutputSingleItem("bbnumden : ","LC =",LC_i);
   }
}
 
template< class P > 
void BBNUMDENTrace3(
   P& FHat_i,
   P& GHat_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputSingleItem("bbnumden : ","FHat =",FHat_i);
   OutputSingleItem("bbnumden : ","GHat =",GHat_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbnumden : black box construction complete" << endl;
   }
}

template< class K > 
void BBNUMDENTrace4(
   K*           Pnts_i,
   unsigned int NumPnts_i,
   K*           ConstPnts_i,
   unsigned int NumYVals_i,
   unsigned int MaxCandidates_i 
   )
{
   if ( PRINTLEVEL > 0 ) {
   cout << "bbnumden : Evalutating black box numerator/denominator" << endl;
   }
   if ( PRINTLEVEL > 3 ) {
   OutputArray("bbnumden : ","Input points","Pnts_i[",Pnts_i,0,NumPnts_i);
   OutputArray("bbnumden : ","Y multipliers","ConstPnts[",
               ConstPnts_i,0,NumPnts_i-1);
   OutputSingleItem("bbnumden : ","NumYVals =",NumYVals_i);
   OutputSingleItem("bbnumden : ","MaxCandidates =",MaxCandidates_i);
   }
}

template< class K, class P > 
void BBNUMDENTrace5(
   P*  FBarArray_i,
   P*  GBarArray_i,
   K*  YArray_i,
   int j_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputArray("bbnumden : ","Y Values","YArray[",YArray_i,0,j_i);
   OutputArray("bbnumden : ","FBar Images","FBarArray[",FBarArray_i,0,j_i);
   OutputArray("bbnumden : ","GBar Images","GBarArray[",GBarArray_i,0,j_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbnumden : Interpolating FBar and GBar" << endl;
   }
}

template< class K > 
void BBNUMDENTrace6(
   K& VON_i,
   K& VOD_i
   )
{
   if ( PRINTLEVEL > 3 ) {
   OutputSingleItem("bbnumden : ","Value of Numerator  = ",VON_i);
   OutputSingleItem("bbnumden : ","Value of Denominaor = ",VOD_i);
   }
   if ( PRINTLEVEL > 1 ) {
   cout << "bbnumden : black box evaluation complete" << endl;
   }
}

void BBNUMDENTrace7(
   int Deg_i,
   int D_i,
   int E_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("bbnumden : ","Working on degree = ",Deg_i);
   OutputSingleItem("bbnumden : ","D = ",D_i);
   OutputSingleItem("bbnumden : ","E = ",E_i);
   }
}

void BBNUMDENTrace8(
   int NumYCandidates_i
   )
{
   if ( PRINTLEVEL > 2 ) {
   OutputSingleItem("bbnumden : ","Working on Y candiate # ",NumYCandidates_i);
   }
}

template< class K, class P > 
void BBNUMDENTrace9(
   int NumVars_i,
   K*  RndPnts_i,
   P&  FHat_i,
   P&  GHat_i,
   K&  ValRatFunc_i,
   K&  ValAprox_i
   )
{
   if ( PRINTLEVEL > 4 ) {
   OutputArray("bbnumden : ","Random Input points","RndPnts_i[",
               RndPnts_i,0,NumVars_i);
   OutputSingleItem("bbnumden : ","FHat = ",FHat_i);
   OutputSingleItem("bbnumden : ","GHat = ",GHat_i);
   OutputSingleItem("bbnumden : ","ValRatFunc = ",ValRatFunc_i);
   OutputSingleItem("bbnumden : ","ValAprox = ",ValAprox_i);
   }
}

void BBNUMDENTrace10( ) {
   if ( PRINTLEVEL > 2 ) {
   cout << "bbnumden : finishing early!" << endl;
   }
}

void BBNUMDEMSGCDTrace1( ) {
   if ( PRINTLEVEL > 2 ) {
      cout << "SpecialExtGCD : Starting" << endl;
   }
}

void BBNUMDEMSGCDTrace2( ) {
   if ( PRINTLEVEL > 2 ) {
      cout << "SpecialExtGCD : Done" << endl;
   }
}

void BBNUMDEMPADETrace1( ) {
   if ( PRINTLEVEL > 2 ) {
      cout << "Pade : Starting" << endl;
   }
}

void BBNUMDEMPADETrace2( ) {
   if ( PRINTLEVEL > 2 ) {
      cout << "Pade : Done" << endl;
   }
}

#endif

#endif
