/*-------------------------------------------------------------
  module: bbbipoly.h
  author: Angel L. Diaz
  date: 8/22/96
-------------------------------------------------------------*/

#ifndef BBBIPOLY
#define BBBIPOLY

/*
#define BBBIPOLY_MAXDEG 100
*/

// for Cauchy
#define BBBIPOLY_MAXDEG 500

#include "BlackBox/Aux/bbdefs.h"

class BiPoly;

/*-------------------------------------------------------------
  class: BiPoly
  date: 8/6/96

  description: A black box compliant univariate polynomial 
  arithmetic wrapper which provides the specialized bivariate 
  structure needed for black box factorization.
-------------------------------------------------------------*/

template < class P > class BiPoly {
   private:
      unsigned int Degree;
      P Coeffs[BBBIPOLY_MAXDEG];
   public:
      inline friend ostream& operator<<( ostream&, const BiPoly<P>& );

      BiPoly( const unsigned int i = 0);
      BiPoly( const BiPoly& );
      ~BiPoly( ) { } 

      inline BiPoly& operator=( const BiPoly& ); 
      inline P& operator[] ( unsigned int );
      inline unsigned int Deg( );
      inline void SetDeg( unsigned int );
};

template < class P > inline ostream& operator<<( 
   ostream&         os, 
   const BiPoly<P>& BiPoly_i 
   ) 
{
   unsigned int i;

   for(i=0;i<=BiPoly_i.Degree;i++)
     os << BiPoly_i.Coeffs[i] << " ";

   return os;
}

template < class P > BiPoly<P>::BiPoly( 
    const unsigned int Degree_i = 0 
    ) : Degree( Degree_i ) 
{ 
   int i;
   MP_INT Zero;
   mpz_init_set_si( &Zero, 0 );
  
   for(i=0;i<BBBIPOLY_MAXDEG;i++) 
      Coeffs[i] = P( &Zero ); 
 
   mpz_clear( &Zero );
}

template < class P > BiPoly<P>::BiPoly( 
   const BiPoly& BiPoly_i 
   ) 
{ 
   *this = BiPoly_i;
}

template < class P > inline BiPoly<P>& BiPoly<P>::operator=( 
   const BiPoly& BiPoly_i 
   ) 
{
   unsigned int i;

   Degree = BiPoly_i.Degree;
   for(i=0;i<=Degree;i++) 
      Coeffs[i] = BiPoly_i.Coeffs[i];

   return *this;  
}

template < class P > inline P& BiPoly<P>::operator[] ( 
   unsigned int Index_i 
   ) 
{
   return Coeffs[Index_i];
}

template < class P > inline unsigned int BiPoly<P>::Deg( ) {
   return Degree;
}

template < class P > inline void BiPoly<P>::SetDeg( 
   unsigned int Degree_i
   ) 
{
   Degree = Degree_i;
}

#endif
