
/*-------------------------------------------------------------
  module: bbselectormpi.h
  author: Angel L. Diaz

  date: 10/28/96

  description: 
     The classes in this module provides for selecting a single 
     value from black box objects that return multiple values.

  classes:
   BlackBoxSelectorMPI

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.
 
-------------------------------------------------------------*/

#ifndef _BBSELECTORMPI
#define _BBSELECTORMPI

#include "BlackBox/Aux/bbselector.h"
#include "BlackBox/Aux/bbdefsmpi.h"

/*-------------------------------------------------------------
  class: BlackBoxSelectorMPI
  date: 10/23/96

  description: 
     Provides for constructing a wrapper for black box objects 
     which returns an array of values from a native imported 
     domain arithmetic (K).

-------------------------------------------------------------*/

template < class K, class B > 
class BlackBoxSelectorMPI : public BlackBoxSelector< K, B > {
   public:
      BlackBoxSelectorMPI( );
      BlackBoxSelectorMPI( B&, int );
      ~BlackBoxSelectorMPI( ) { }

      inline void Distribute( int,  K*, int&, int& );
      inline K    Wait( int, int& );
      inline void Kill( int, int& );
};

template < class K, class B > 
BlackBoxSelectorMPI< K, B >::BlackBoxSelectorMPI( ) {
}

template < class K, class B > 
BlackBoxSelectorMPI< K, B >::BlackBoxSelectorMPI(
   B&  BB_i,
   int Index_i
   ) : BlackBoxSelector< K, B >( BB_i, Index_i )
{
}

template < class K, class B > 
inline void BlackBoxSelectorMPI< K, B >::Distribute(
   int Dest_i,
   K* Pnts_i,
   int& IsDist_m,
   int& Status_m
   )
{
  (*BB).Distribute( Dest_i, Pnts_i, IsDist_m, Status_m );
}

template < class K, class B >
inline K BlackBoxSelectorMPI< K, B >::Wait(
   int  Source_i,
   int& Status_m
   )
{
   return  ((*BB).Wait( Source_i, Status_m ))[ Index ];
}

template < class K, class B >
inline void BlackBoxSelectorMPI< K, B >::Kill(
   int  Dest_i,
   int& Status_m
   )
{
   (*BB).Kill( Dest_i, Status_m );
}

#endif
