
/*-------------------------------------------------------------
  module: bbtime.h
  author: Angel L. Diaz

  date: 10/30/96

  description: 
     The function in this module provide for benchmarking
     resource usage.

  functions:
     GetCPUTime

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.

-------------------------------------------------------------*/

#ifndef _BBTIME
#define _BBTIME

#include <stdio.h>
#include <unistd.h>
#include <time.h>
#include <sys/times.h>
#include <sys/param.h>

/*-------------------------------------------------------------
  function: GetCPUTime 
  Original Author: Austin Lobo 

  date: 10/30/96

  description: 
    Get the elapsed and cpu and user times for current process. 
    Print them iff get_cputime was initialized by a (dummy) 
    call earlier.

-------------------------------------------------------------*/

void GetCPUTime( char *task )
{
   static char hostname[128];
   struct tms  buffer;
   static struct tms ibuffer;
   char *pp;
   long    hms, mms, sms, istime, iutime;
   float utime, stime,rtime; 
   static int  init_etime = 0;
  
   if ( init_etime == 0)  {
      times(&ibuffer);  
      gethostname( hostname, 128);
      pp = hostname;
      while (*pp != '.' && *pp !='\0' ) ++pp;
      *pp = '\0';
      init_etime = 1;
  }
  else { 
      /*  print elapsed usage on second and subseq. calls 
       *  note HZ is in sys/param.h and defines the no. of 
       *  ticks per clock
       */

      times( &buffer );
      istime =  buffer.tms_stime - ibuffer.tms_stime;
      iutime =  buffer.tms_utime - ibuffer.tms_utime;
      stime  =  ((float) (istime))/((float)HZ);
      utime  =  ((float) (iutime))/((float)HZ);
      mms    = (istime+iutime)/HZ;
      hms    = mms/3600;
      mms    = ( mms - hms * 3600)/60;
      sms    = mms % 60;
      printf( "%s %11.1f User %11.1f Sys \t%d:%d:%d Total (on %s)\n", 
             task, utime, stime, hms, mms, sms, hostname);
  }
  return;
}

void GetCPUTime( ofstream& LogFile_m )
{
   static char hostname[128];
   struct tms  buffer;
   static struct tms ibuffer;
   char *pp;
   long    hms, mms, sms, istime, iutime;
   float utime, stime,rtime; 
   static int  init_etime = 0;
  
   if ( init_etime == 0)  {
      times(&ibuffer);  
      gethostname( hostname, 128);
      pp = hostname;
      while (*pp != '.' && *pp !='\0' ) ++pp;
      *pp = '\0';
      init_etime = 1;
  }
  else { 
      /*  print elapsed usage on second and subseq. calls 
       *  note HZ is in sys/param.h and defines the no. of 
       *  ticks per clock
       */

      times( &buffer );
      istime =  buffer.tms_stime - ibuffer.tms_stime;
      iutime =  buffer.tms_utime - ibuffer.tms_utime;
      stime  =  ((float) (istime))/((float)HZ);
      utime  =  ((float) (iutime))/((float)HZ);
      mms    = (istime+iutime)/HZ;
      hms    = mms/3600;
      mms    = ( mms - hms * 3600)/60;
      sms    = mms % 60;
      LogFile_m << utime << " User " << stime << " Sys \t" <<  hms << ":" 
                << mms << ":" << sms << " (on " << hostname << " )" << endl;
  }
  return;
}

#endif

