
/*-------------------------------------------------------------
  module: blackbox.h
  author: Angel L. Diaz

  date: 10/21/96

  description: 
    The classes in this module serve as the base classes for all 
    black box objects and extended domain black box objects. These
    classes define the minimum interface required by all derived 
    black boxes.

  classes: 
     BlackBox
     BlackBoxEx

  (c) Copyright 1996 Angel Diaz and Erich Kaltofen, All Rights Reserved
  This code is provided WITHOUT WARRANTY either expressed or implied.

-------------------------------------------------------------*/

#ifndef _BLACKBOX
#define _BLACKBOX

#include "BlackBox/Aux/bbdefs.h"

/*-------------------------------------------------------------
  class: BlackBox
  date: 10/21/96

  description: 
     Provides a base class for black box objects.

-------------------------------------------------------------*/

template< class K > 
class BlackBox {
   protected:
      int          Degree;
      unsigned int NumVariables;
      double       Probability;
   public:
      BlackBox( );
      ~BlackBox( ) { }
      virtual inline K* operator() ( K* ) { }
      inline unsigned int Deg( );
      inline unsigned int NumVars( );
      inline double Prob( );
};

template< class K > 
BlackBox< K >::BlackBox( ) {
   Degree       = 0;
   Probability  = 0;
   NumVariables = 0;
}

template< class K > 
inline unsigned int BlackBox< K >::Deg( ) { 
   return Degree;        
}

template< class K > 
inline unsigned int BlackBox< K >::NumVars( ) { 
   return NumVariables; 
}

template< class K > 
inline double BlackBox< K >::Prob( ) { 
   return Probability;  
} 

/*-------------------------------------------------------------
  class: BlackBoxEx
  date: 10/10/96
 
  description: 
     Provides a base class for extended domain black box objects.

-------------------------------------------------------------*/

template< class K, class L > 
class BlackBoxEx : public BlackBox< K > {
   public:
      BlackBoxEx( ) : BlackBox< K >( ) { }
      ~BlackBoxEx( ) { }
      virtual inline L* operator() ( L* ) { }
};

#endif
